/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphyItem } from "@saga-web/graphy/lib";
import {
    SColor,
    SPainter,
    SPath2D,
    SPoint,
    SPolygonUtil,
    SRect
} from "@saga-web/draw/lib";
import { Zone } from "../types/Zone";
import { SMouseEvent } from "@saga-web/base/lib";

/**
 * 业务空间item
 *
 * @author  郝建龙
 */
export class ZoneItem extends SGraphyItem {
    /** 空间所有数据  */
    data: Zone;
    /** 不可选标志  */
    unselectFlag = false;
    /** 空间轮廓线坐标list  */
    private readonly pointArr: SPoint[][][] = [];
    /** X坐标最小值  */
    private minX = 0;
    /** X坐标最大值  */
    private maxX = 0;
    /** Y坐标最小值  */
    private minY = 0;
    /** Y坐标最大值  */
    private maxY = 0;
    /** path    */
    private pathList: SPath2D[] = [];
    /** 点击位置坐标  */
    private clickPoint: SPoint | undefined;
    /** 选中时的颜色  */
    private selectColor = new SColor("#1abc9c");
    /** 不可选时的颜色  */
    private unselectColor = new SColor("#cecece");

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      空间数据
     */
    constructor(parent: SGraphyItem | null, data: Zone) {
        super(parent);
        this.data = data;
        if (this.data.OutLine.length && this.data.OutLine[0].length) {
            let tempArr = this.data.OutLine;
            this.minX = tempArr[0][0][0].X;
            this.maxX = this.minX;
            this.minY = -tempArr[0][0][0].Y;
            this.maxY = this.minY;
            // 处理轮廓点数组,同时计算最大最小值
            this.pointArr = tempArr.map(t => {
                let spath = new SPath2D();
                let tempArr = t.map(it => {
                    let array = it.map(item => {
                        let x = item.X,
                            y = -item.Y;
                        if (x < this.minX) {
                            this.minX = x;
                        }
                        if (y < this.minY) {
                            this.minY = y;
                        }
                        if (x > this.maxX) {
                            this.maxX = x;
                        }
                        if (y > this.maxY) {
                            this.maxY = y;
                        }
                        return new SPoint(x, y);
                    });
                    spath.polygon(array);
                    return array;
                });
                this.pathList.push(spath);
                return tempArr;
            });
        }
    } // Constructor

    /**
     * Item对象边界区域
     *
     * @return SRect
     */
    boundingRect(): SRect {
        return new SRect(
            this.minX,
            this.minY,
            this.maxX - this.minX,
            this.maxY - this.minY
        );
    } // Function boundingRect()

    /**
     * 鼠标单击事件
     *
     * @param   event   事件参数
     * @return  boolean
     */
    onClick(event: SMouseEvent): boolean {
        if (!this.unselectFlag && this.selectable) {
            this.selected = !this.selected;
            this.clickPoint = new SPoint(event.x, event.y);
            this.$emit("click", event);
        }
        return true;
    } // Function onClick()

    /**
     * 判断点是否在区域内
     *
     * @param x
     * @param y
     */
    contains(x: number, y: number): boolean {
        // return true;
        for (let j = 0; j < this.pointArr.length; j++) {
            let arr = this.pointArr[j];
            if (arr.length < 1 || !SPolygonUtil.pointIn(x, y, arr[0])) {
                continue;
            }
            if (arr.length == 1) {
                return true;
            }
            let flag = false;
            for (let i = 1; i < arr.length; i++) {
                if (SPolygonUtil.pointIn(x, y, arr[i])) {
                    flag = true;
                    break;
                }
            }
            if (flag) {
                continue;
            }
            return true;
        }
        return false;
    } // Function contains()

    /**
     * Item绘制操作
     *
     * @param   painter painter对象
     * @param   rect    绘制区域
     */
    onDraw(painter: SPainter, rect?: SRect): void {
        painter.pen.color = SColor.Transparent;
        if (this.unselectFlag) {
            painter.brush.color = this.unselectColor;
        } else {
            painter.brush.color = this.selected
                ? this.selectColor
                : this.data.Color;
        }
        painter.pen.lineWidth = 200;
        this.pathList.map(t => {
            painter.drawPath(t);
        });
        painter.brush.color = SColor.Black;
        painter.font.size = painter.toPx(10);
        if (this.clickPoint) {
            painter.drawText(
                this.data.RoomLocalName,
                this.clickPoint.x,
                this.clickPoint.y
            );
        }
    } // Function onDraw()
} // Class ZoneItem
